#include "DIALOG.h"

#ifndef WIN32
  #define EM_FILE 1
#endif

#ifdef WIN32
  #include "windows.h"
#elif (EM_FILE == 1)
  #include "FS.h"
  #include "stdlib.h"
#else
#endif

/*********************************************************************
*
*     Defines
*
**********************************************************************
*/
/*********************************************************************
*
*     Static data
*
**********************************************************************
*/
static char _acBuffer[0x1000];



/*********************************************************************
*
*     Static code
*
**********************************************************************
*/
/*********************************************************************
*
*       APP_GetData
*/
int APP_GetData(void * p, const U8 * * ppData, unsigned NumBytesReq, U32 Off){
#ifdef WIN32
  DWORD    NumBytesRead;
  HANDLE * phFile;

  phFile = (HANDLE *)p;
  //
  // Check buffer size
  //
  if (NumBytesReq > sizeof(_acBuffer)) {
    NumBytesReq = sizeof(_acBuffer);
  }
  //
  // Set file pointer to the required position
  //
  SetFilePointer(*phFile, Off, 0, FILE_BEGIN);
  //
  // Read data into buffer
  //
  ReadFile(*phFile, _acBuffer, NumBytesReq, &NumBytesRead, NULL);
  //
  // Set data pointer to the beginning of the buffer
  //
  *ppData = _acBuffer;
  //
  // Return number of available bytes
  //
  return NumBytesRead;
#elif (EM_FILE == 1)
  U32          NumBytesRead;
  U32          ReadBytes;
  static U8    aBuffer[4096];
  FS_FILE    * pFile;

  pFile = (FS_FILE *)p;
  //
  // Set file pointer to the offset position
  //
  FS_SetFilePos(pFile, Off, FS_FILE_BEGIN);
  //
  // Check that we don#t read more than the buffer can hold
  //
  if (NumBytesReq <= sizeof(aBuffer)) {
    ReadBytes = NumBytesReq;
  } else {
    ReadBytes = sizeof(aBuffer);
  }
  //
  // read from file
  //
  NumBytesRead = FS_Read(pFile, (void *)aBuffer, ReadBytes);
  //
  // Set pointer to the buffer address
  //
  *ppData = aBuffer;
  return NumBytesRead;
#else
#endif
}

/*********************************************************************
*
*       _ShowGIFEx
*/
void _ShowGIFEx(void * pFile) {
  GUI_GIF_IMAGE_INFO ImageInfo = {0};  //Info structure of one particular GIF image of the GIF file
  GUI_GIF_INFO       GifInfo   = {0};  //Info structure of GIF file
  int                i;
  int                XPos;
  int                YPos;
  int                aTime[30];
  
  
    
  XPos = 0;
  YPos = 0;
  if (pFile) {
    GUI_GIF_GetInfoEx(APP_GetData, pFile, &GifInfo);  //Get GIF info structure
    //
    // Endless loop for test purpose
    //
    XPos = (LCD_GetXSize() - GifInfo.xSize) / 2;
    YPos = (LCD_GetYSize() - GifInfo.ySize) / 2;
    while (1) {
      XPos = (LCD_GetXSize() - GifInfo.xSize) / 2;
      for (i = 0; i < GifInfo.NumImages; i++) {
        //
        // Draw image with index i
        //
        aTime[i] = GUI_GetTime();
        GUI_GIF_DrawSubEx(APP_GetData, pFile, XPos, YPos, i);
        aTime[i] = GUI_GetTime() - aTime[i];
        GUI_GIF_GetImageInfoEx(APP_GetData, pFile, &ImageInfo, i);
        //
        // Delay with period of the image with index i
        //
//        GUI_Delay(ImageInfo.Delay ? ImageInfo.Delay : 500);
        GUI_Delay(20);
      }
    }
  }
}

/*********************************************************************
*
*     Public code
*
**********************************************************************
*/
/*********************************************************************
*
*     MainTask
*/
void MainTask(void) {
#ifdef WIN32
  HANDLE hFile;
  //
  // Enter a path to a gif
  //
  char   sFilename[] = "c:\\AnimatedGif_100x100.gif";
  GUI_MEMDEV_Handle * pMem;
  GUI_MEMDEV_Handle * pMem0;
  GUI_GIF_INFO       GifInfo   = {0};
  GUI_GIF_IMAGE_INFO ImageInfo = {0};
  int                Run;
  int                i;
#elif (EM_FILE == 1)
  int    i;
  char   sFilename[] = "AnimatedGif_100x100.gif";
  static char         acVolumeName[10];
  FS_FILE           * pFile;
  GUI_MEMDEV_Handle * pMem;
  GUI_MEMDEV_Handle * pMem0;
  GUI_GIF_INFO       GifInfo   = {0};
  GUI_GIF_IMAGE_INFO ImageInfo = {0};
  int                Run;
#else
#endif

  GUI_Init();
  GUI_SetBkColor(GUI_WHITE);
  GUI_Clear();
#ifdef WIN32
  //
  // Open a file
  //
  hFile = 0;
  hFile = CreateFile(sFilename, GENERIC_READ, 0, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
  if (hFile) {
    //
    // Get the number of images
    //
    GUI_GIF_GetInfoEx(APP_GetData, &hFile, &GifInfo);
    //
    // Allocate memory to save NumImages pointer to memory device handles
    //
    pMem0 = pMem = (U32 *)malloc(GifInfo.NumImages * sizeof(GUI_MEMDEV_Handle));
    //
    // Draw each sub image into one memory device
    //
    for (i = 0; i < GifInfo.NumImages; i++) {
      *pMem = GUI_MEMDEV_CreateFixed32(0, 0, GifInfo.xSize, GifInfo.ySize);
      GUI_MEMDEV_Select(*pMem);
      GUI_GIF_DrawSubEx(APP_GetData, &hFile, 0, 0, i);
      pMem++;
    }
    GUI_MEMDEV_Select(0);
    //
    // Get the delay, in this case we use the delay of the first image only.
    // Maybe it is better to get all delays at once and store them in an array.
    //
    GUI_GIF_GetImageInfoEx(APP_GetData, &hFile, &ImageInfo, 0);
    Run = 1;
    //
    // Let pMem point to the first memory device handle and draw all memory devices
    //
    while (Run) {
      pMem = pMem0;
      for (i = 0; i < GifInfo.NumImages; i++) {
        GUI_MEMDEV_Write(*pMem);
        pMem++;
        GUI_Delay(ImageInfo.Delay);
      }
    }
    //
    // Delete memory devices, free memory and close handle
    //
    pMem = pMem0;
    for (i = 0; i < GifInfo.NumImages; i++) {
      GUI_MEMDEV_Delete(*pMem++);
    }
    free(pMem0);
    CloseHandle(hFile);
  }
#elif (EM_FILE == 1)
  FS_Init();
  FS_FAT_SupportLFN();
  FS_GetVolumeName(0, acVolumeName, sizeof(acVolumeName));
    if (FS_Mount(acVolumeName) > 0) {
      pFile = FS_FOpen(sFilename, "r");
      GUI_GIF_GetInfoEx(APP_GetData, pFile, &GifInfo);
      pMem0 = pMem = (U32 *)malloc(GifInfo.NumImages * sizeof(GUI_MEMDEV_Handle));
      for (i = 0; i < GifInfo.NumImages; i++) {
        *pMem = GUI_MEMDEV_CreateFixed32(0, 0, GifInfo.xSize, GifInfo.ySize);
        GUI_MEMDEV_Select(*pMem);
        GUI_GIF_DrawSubEx(APP_GetData, pFile, 0, 0, i);
        pMem++;
      }
      GUI_MEMDEV_Select(0);
      GUI_GIF_GetImageInfoEx(APP_GetData, pFile, &ImageInfo, 0);
      Run = 1;
      while (Run) {
        pMem = pMem0;
        for (i = 0; i < GifInfo.NumImages; i++) {
          GUI_MEMDEV_Write(*pMem);
          pMem++;
          GUI_Delay(ImageInfo.Delay);
        }
      }
      pMem = pMem0;
      for (i = 0; i < GifInfo.NumImages; i++) {
        GUI_MEMDEV_Delete(*pMem++);
      }
      free(pMem0);
      FS_FClose(pFile);
    }
#else
#endif
  while (1) {
    GUI_Delay(100);
  }
}

/*************************** End of file ****************************/